
# Community Accounting module for Drupal

For an architectural description see: https://matslats.net/mutual-credit-architecture-5

For a full description of the module, visit the [project page](https://www.drupal.org/project/mutual_credit).

Submit bug reports and feature suggestions, or track changes in the [issue queue](https://www.drupal.org/project/issues/mutual_credit).

**WARNING!**

The intention behind this module is to foster a culture of experimentation, not 
to impose ideological constraints about how   a money system should work. 
Consequently it can be used as straight mutual credit, as the name implies, or 
for fiat or asset currencies in which units are 'issued' into circulation from 
an account allocated for that purpose. Nonetheless, fools rush in where angels 
fear to tread! Many lessons have already been learned about currency design and 
the process of encouraging communities to adopt them. A badly managed accounting 
system can cause people to lose out and create bad feeling and resistance to 
future innovation, especially because few people understand how money actually 
works.

## Installation

Enable Mutual Credit module. You may also want Field UI to add fields to transactions or wallets.
  
Two wallets are created by default so the transaction forms will work immediately; both belong to user 1; The first is named after user 1, the second is called 'Spare'

Visit `admin/accounting/misc` and specify the maximum number of wallets users can hold (except user 1). Choose which view to show as a transactions tab on the wallet page.

Configure how the unit of account is formatted by specifying the prefix, separator, divisor, and fractions and suffix in a single string. For example to show hours and quarters of an hour put
H0:59/4m

Visit `admin/accounting/transaction/workflows` and design the workflows. Note than workflows inherited from a credit commons node can only be translated, not edited. Each workflow generates a default transaction form, visible at transact/.

Visit `admin/accounting/transactions/fields` to add and manage fields. Notice that you cant do much with transaction form Displays.

Add fields to wallets at `admin/accounting/wallets`

### Form Designer
To make more attractive and user-friendly forms, enable the Form Designer module. On `admin/accounting/forms` you can now create a new transaction form. Choose the desired workflow, path, menu link, field defaults, and then design the form as a Twig template.

### Limits
Most currencies have minimum and sometimes maximum wallet balance limits. This module adds a validation to each transaction to ensure that the payee wallet would not exceed its max limit nor the payer exceed its min limit. There are default limits on `admin/accounting/misc` which can be overridden per-wallet on `wallet/99/manage`. Limits can be determined in different ways. See the module's own README.md about the limits plugin.

### Mass transactions
This module provides two transaction forms in which either the payer or the payee is actually multiple wallets.  It creates a single transaction with an entry for each payer or payee. The transaction forms allow the user to specify the 2nd parties inclusively or exclusively and to mail all the other parties. Mass transactions cannot use arbitrary workflows and generally should enter into the finished state immediately.

### Credit Commons

[Credit Commons](http://creditcommons.net) is a protocol for reading and writing mutual credit over a rest 
API, but more relevantly here, it also helps groups to trade with each other. External intertrading happens between web servers via a 3rd party web service. Currently the only such service available is Clearing Central, provided by CES.
To configure the module you need to know the url of the trunkward node, and give your drupal site an alphanumeric name as a child of that trunkward node. You must also specify the exchange rate and a special wallet to handle external trade. That wallet can have its own balance limits.

## Advanced Tips

### Transaction Form tabs/tasks.
The system automatically generates one form for every local workflow and creates local tasks for them under 'transact/' or 'Built-in'.
The mcapi_forms module allows its forms to have an autogenerated tab under transact'.
The mcapi_cc module creates a new tab, transact/remote, and puts remote forms under there.
The mcapi_mass module provides two mass transaction forms, under transact/mass
```Built-in:
  - One form
  - for every 
  - local workflow
Bill (Designed form)
Credit (Designed form)
Mass:
  - One to many
  - Many to one
Remote:
  - One form
  - for every
  - remote workflow
```

### Transaction entity creation and editing.
To create a transaction requires knowing the workflow in advance. Every transaction form is based on a workflow and creates an empty transaction when the form loads (a bit like the node->type) When the form is submitted the transaction fields are populated, and the transaction is 'assembled' via the Transaction Subscriber. Assembly means that new entries can be added by other modules. Then the transaction is validated like a normal entity and saved to the user temp store while the confirmation page is displayed. The confirmation form is actually a form for changing the state of the transaction from 'validated' to the first transaction state defined in the workflow. The TransactionOperationForm is used whenever the transaction changes state. Then the transaction is saved as revision 1. The workflow defines which users can do which operations to a transaction in its current state, and when the user does an operation, the same TransactionOperationForm is used.

### Views integration
As well as the main transaction entity storage there are 2 other db tables with copies of the transaction which are intended for the views module to read.
- The transactions_index table has two rows for every transaction
- The transaction_stats table has a row for every wallet.

## Maintainers

- Matthew Slater - https://www.drupal.org/u/matslats
